class MDM(object):
    """
    Base MDM class

    Workspace ONE UEM REST APIs allows you to manage all the functionalities of Mobile Device Management (MDM).
    The functionalities that are included but not limited to are device commands,
    retrieval of compliance, profile, network, location, and event log details. 
    """
    def __init__(self, client):
        self.client = client

    def _get(self, module='mdm', path=None, version=None, params=None,
             header=None):
        """GET requests for base mdm endpoints"""
        return self.client.get(module=module, path=path,
                               version=version, params=params, header=header)

    def _post(self, module='mdm', path=None, version=None, params=None,
              data=None, json=None, header=None):
        """POST requests for base mdm endpoints"""
        return self.client.post(module=module, path=path, version=version,
                                params=params, data=data,
                                json=json, header=header)

    def _put(self, module='mdm', path=None, version=None, params=None,
             data=None, json=None, header=None):
        """PUT requests for base mdm endpoints"""
        return self.client.put(module=module, path=path, version=version,
                               params=params, data=data,
                               json=json, header=header)

    def _patch(self, module='mdm', path=None, version=None, params=None,
               data=None, json=None, header=None):
        """Patch requests for base mdm endpoints"""
        return self.client.patch(module=module, path=path, version=version,
                                 params=params, data=data,
                                 json=json, header=header)

    def _delete(self, module='MDM', path=None, version=None, params=None,
                data=None, json=None, header=None):
        return self.client.delete(module=module, path=path, version=version,
                                  params=params, data=data, json=json, header=header)



class WorkspaceOneAPIError(Exception):
    def __init__(self, json_response=None):
        if json_response is None:
            pass
        else:
            self.response = json_response
            self.error_code = json_response.get('errorCode')
            self.error_msg = str(json_response.get('message'))
            if self.error_code is None:
                self.error_code = 0
                self.error_msg = 'Unknown API error occurred'

    def __str__(self):
        return 'Error #{}: {}'.format(self.error_code, self.error_msg)



from .system import System


class Users(System):

    def __init__(self, client):
        System.__init__(self, client)

    def search(self, **kwargs):
        """
        Returns the Enrollment User's details matching the search parameters

        /api/system/users/search?{params}

        PARAMS:
            username={username}
            firstname={firstname}
            lastname={lastname}
            email={email}
            organizationgroupid={locationgroupid}
            role={role}
        """
        return System._get(self, path='/users/search', params=kwargs)

    def get_user_by_uuid(self, uuid):
        """
        Returns the enrollment user for a specific uuid using the v2 endpoint.

        PARAMS:
            uuid (str): AirWatch UUID to return
        """
        _path = "/users/{}".format(uuid)
        _header = {'Accept': 'application/json;version=2'}
        return System._get(self, header=_header, path=_path)

    def create_user(self, **kwargs):
        """
        Create an enrollment user using the v2 endpoint.

        /api/system/users/

        PARAMS:
            externalId={e559e7df-4ba0-4891-9fcd-8574c1770d34}
            userName={username}
            password={password}
            firstName={firstname}
            lastName={lastname}
            displayName={displayname}
            userPrincipalName={testuser@gandalf.dev}
            emailAddress={noreply@vmware.com}
            emailUsername={noreply@vmware.come}
            phoneNumber={1-111-111-1111}
            mobileNumber={+1(111)-111-1111}
            messageType={Email}
            messageTemplateUuid={53f732a0-3e08-474d-80f4-ff976b8eb698}
            enrollmentRoleUuid={599b9117-f399-4e60-a96b-cd1f771d5f06}
            status=true,
            securityType={directory}     REQUIRED
            deviceStagingEnabled=false,
            deviceStagingType={StagingDisabled}
            organizationGroupUuid={6fbc95c6-3269-4a88-804d-c8db7f479d7f}
            enrollmentOrganizationGroupUuid={94b1b965-59b9-462c-ad18-4a228f9830dd}
            customAttribute1={CustomAttribute1}
            customAttribute2={CustomAttribute2}
            customAttribute3={CustomAttribute3}
            customAttribute4={CustomAttribute4}
            customAttribute5={CustomAttribute5}
            aadMappingAttribute={e559e7df-4ba0-4891-9fcd-8574c1770d34}
            department={Sales}
            employeeIdentifier={12345}
            costCenter={110)
        """
        _header = {'Accept': 'application/json;version=2'}
        return System._post(self, header=_header, path="/users/", json=kwargs)

    def update_user_by_uuid(self, uuid: str=None, **kwargs):
        """
        Update the enrollment user with attributes using the v2 endpoint.

        /api/system/users/{uuid}

        PARAMS:
            password={Password1}
            firstName={Firstname}
            lastName={Lastname}
            displayName={displayName}
            userPrincipalName={testuser@gandalf.dev}
            emailAddress={noreply@vmware.com}
            emailUsername={noreply@vmware.com}
            phoneNumber={5551234567}
            mobileNumber={5551234567}
            messageType={Email}
            messageTemplateUuid={2aca918b-3468-4539-8750-41a7074b120d}
            deviceStagingEnabled": false,
            deviceStagingType={StagingDisabled}
            enrollmentRoleUuid={0aa0256e-89c6-450e-854c-aa97233b61b6}
            enrollmentOrganizationGroupUuid={db1d3802-6885-4035-99ab-e39239b5a0f2}
            CustomAttribute1={CustomAttribute1}
            CustomAttribute2={CustomAttribute2}
            CustomAttribute3={CustomAttribute3}
            CustomAttribute4={CustomAttribute4}
            CustomAttribute5={CustomAttribute5}
        """
        _path = "/users/{}".format(uuid)
        _header = {'Accept': 'application/json;version=2'}
        return System._put(self, path=_path, header=_header, json=kwargs)

    def delete_user_by_uuid(self, uuid):
        """
        Delete the enrollment user by enrollment user uuid with the v2 endpoint.

        :param uuid:
        :return: API response
        """
        _path = '/users/{}'.format(uuid)
        _header = {'Accept': 'application/json;version=2'}
        return System._delete(self, header=_header, path=_path)

    def delete_user_by_id(self, user_id):
        """
        Delete the enrollment user by enrollment user id

        :param user_id:
        :return: API response
        """
        path = '/users/{}/delete'.format(user_id)
        return System._delete(self, path=path)

    def create_device_registration_to_user(self, user_id, register_device_details):
        path = '/users/{}/registerdevice'.format(user_id)
        response = System._post(path=path, data=register_device_details)
        return response


class MEM(object):
    """
    Base MEM class

    Workspace ONE UEM REST APIs enables you to manage emails on devices. 
    Through these APIs, you can retreive email management device details 
    and apply compliance policies to devices.
    """
    pass


# TODO: Implement


class System(object):
    """
    Base System class

    Workspace ONE UEM REST APIs allows you to manage all the core functionalities around console administration where you can:
    - Create and assign administrator accounts so admins can easily manage users and devices.
    - Create, manage, and view device enrollment user's.
    Identify users and establish permissions using organization groups.
    - Group sets of users into user groups which act as filters (in addition to organization groups) for assigning profiles and applications.
    - Extract particular values from the devices and return it to Workspace ONE UEM. These attributes can then be associated with other rules to further assign to the devices.
    """
    def __init__(self, client):
        self.client = client

    def _get(self, module='system', path=None,
             version=None, params=None, header=None):
        """
        GET requests for base system endpoints
        """
        return self.client.get(module=module, path=path,
                               version=version, params=params, header=header)

    def _post(self, module='system', path=None,
              version=None, params=None, data=None, json=None, header=None):
        """POST requests"""
        return self.client.post(module=module, path=path, version=version,
                                params=params, data=data,
                                json=json, header=header)

    def _post_no_error_check(self, module='system', path=None,
                             version=None, params=None, data=None,
                             json=None, header=None):
        """POST requests with no error check when none json is returned"""
        return self.client.post_no_error_check(module=module, path=path,
                                               version=version, params=params,
                                               data=data,
                                               json=json, header=header)

    def _put(self, module='system', path=None,
             version=None, params=None, data=None, json=None, header=None):
        """PUT requests"""
        return self.client.put(module=module, path=path, version=version,
                               params=params, data=data,
                               json=json, header=header)

    def _delete(self, module="system", path=None,
                version=None, params=None, header=None):
        """Delete requests"""
        return self.client.delete(
            module=module, path=path, version=version, params=params, header=header
        )



from __future__ import print_function, absolute_import
import base64
import logging
import requests
from pyws1uem.error import WorkspaceOneAPIError
from pyws1uem.mdm.devices import Devices
from pyws1uem.system.groups import Groups
from pyws1uem.system.users import Users
from pyws1uem.system.system import System

# Enabling debugging at http.client level (requests->urllib3->http.client)
# you will see the REQUEST, including HEADERS and DATA, and RESPONSE with
# HEADERS but without DATA.
# the only thing missing will be the response.body which is not logged.
try:
    from http.client import HTTPConnection
except ImportError:
    from httplib import HTTPConnection
HTTPConnection.debuglevel = 0

# TODO: programing using library should be able to set logging level
# TODO: Implement logging to using config https://docs.python.org/3/howto/logging.html#configuring-logging
# TODO: sett logging correclty for a library https://docs.python.org/3/howto/logging.html#configuring-logging-for-a-library
logging.basicConfig()
logging.getLogger().setLevel(logging.DEBUG)
requests_log = logging.getLogger("requests.packages.urllib3")
requests_log.setLevel(logging.DEBUG)
requests_log.propagate = True


class WorkspaceOneAPI(object):
    def __init__(self, env: str, apikey: str, username: str, password: str):
        """
        Initialize an AirWatchAPI Client Object.

        :param  env: Base URL of the AirWatch API Service
                apikey: API Key to authorize
                username: Admin username
                password: corresponding pasword
        """
        self.env = env
        self.apikey = apikey
        self.username = username
        self.password = password
        self.groups = Groups(self)
        self.devices = Devices(self)
        self.users = Users(self)
        self.system = System(self)

    def get(self, module, path, version=None, params=None, header=None, timeout=30):
        """
        Sends a GET request to the API. Returns the response object.
        """
        if header is None:
            header = {}
        header.update(
            self._build_header(self.username, self.password, self.apikey, header)
        )
        header.update({"Content-Type": "application/json"})
        endpoint = self._build_endpoint(self.env, module, path, version)
        try:
            r = requests.get(endpoint, params=params, headers=header, timeout=timeout)
            r = self._check_for_error(r)
            return r
        except WorkspaceOneAPIError as e:
            raise e

    def post(
        self,
        module,
        path,
        version=None,
        params=None,
        data=None,
        json=None,
        header=None,
        timeout=30,
    ):
        """
        Sends a POST request to the API. Returns the response object.
        """
        if header is None:
            header = {}
        header.update(
            self._build_header(self.username, self.password, self.apikey, header)
        )
        endpoint = self._build_endpoint(self.env, module, path, version)
        try:
            r = requests.post(
                endpoint,
                params=params,
                data=data,
                json=json,
                headers=header,
                timeout=timeout,
            )
            r = self._check_for_error(r)
            return r
        except WorkspaceOneAPIError as e:
            raise e

    def put(
        self,
        module,
        path,
        version=None,
        params=None,
        data=None,
        json=None,
        header=None,
        timeout=30,
    ):
        """
        Sends a PUT request to the API. Returns the response object.
        """
        if header is None:
            header = {}
        header.update(
            self._build_header(self.username, self.password, self.apikey, header)
        )
        endpoint = self._build_endpoint(self.env, module, path, version)
        try:
            r = requests.put(
                endpoint,
                params=params,
                data=data,
                json=json,
                headers=header,
                timeout=timeout,
            )
            r = self._check_for_error(r)
            return r
        except WorkspaceOneAPIError as e:
            raise e

    def patch(
        self,
        module,
        path,
        version=None,
        params=None,
        data=None,
        json=None,
        header=None,
        timeout=30,
    ):
        """
        Sends a Patch request to the API. Returns the response object.
        """
        if header is None:
            header = {}
        header.update(
            self._build_header(self.username, self.password, self.apikey, header)
        )
        endpoint = self._build_endpoint(self.env, module, path, version)
        try:
            r = requests.patch(
                endpoint,
                params=params,
                data=data,
                json=json,
                headers=header,
                timeout=timeout,
            )
            r = self._check_for_error(r)
            return r
        except WorkspaceOneAPIError as e:
            raise e

    # NOQA

    def delete(
        self,
        module,
        path,
        version=None,
        params=None,
        data=None,
        json=None,
        header=None,
        timeout=30,
    ):
        """
        Sends a DELETE request to the API. Returns the response object.
        """
        if header is None:
            header = {}
        header.update(
            self._build_header(self.username, self.password, self.apikey, header)
        )
        endpoint = self._build_endpoint(self.env, module, path, version)
        try:
            r = requests.delete(
                endpoint,
                params=params,
                data=data,
                json=json,
                headers=header,
                timeout=timeout,
            )
            r = self._check_for_error(r)
            return r
        except WorkspaceOneAPIError as e:
            raise e

    @staticmethod
    def _check_for_error(response):
        """
        Checks the response for json data, then for an error, then for
        a status code
        """
        if response.headers.get("Content-Type") in (
            "application/json",
            "application/json; charset=utf-8",
        ):
            json = response.json()
            if json.get("errorCode"):
                raise WorkspaceOneAPIError(json_response=json)
            else:
                return json
        else:
            return response.status_code

    @staticmethod
    def _build_endpoint(base_url, module, path=None, version=None):
        """
        Builds the full url endpoint for the API request
        """
        if not base_url.startswith("https://"):
            base_url = "https://" + base_url
        if base_url.endswith("/"):
            base_url = base_url[:-1]
        if version is None:
            url = "{}/api/{}".format(base_url, module)
        else:
            url = "{}/api/v{}/{}".format(base_url, version, module)
        if path:
            if path.startswith("/"):
                return url + "{}".format(path)
            else:
                return url + "/{}".format(path)
        return url

    @staticmethod
    def _build_header(username, password, token, header=None):
        """
        Build the header with base64 login, AW API token,
        and accept a json response
        """
        if not header:
            header = {}
        hashed_auth = base64.b64encode(
            (username + ":" + password).encode("utf8")
        ).decode("utf-8")
        header.update({"Authorization": "Basic {}".format(hashed_auth)})
        header.update({"aw-tenant-code": token})
        if not header.get("Accept"):
            header.update({"Accept": "application/json"})
        return header



from .mdm import MDM


class Devices(MDM):
    """
    A class to manage functionalities of Mobile Device Management (MDM).
    """

    def __init__(self, client):
        MDM.__init__(self, client)

    def search(self, **kwargs):
        """Returns the Device information matching the search parameters."""
        return MDM._get(self, path='/devices', params=kwargs)

    def searchv2(self, **kwargs):
        """Returns the Device information matching the search parameters with v2 endpoint."""
        _header = {'Accept': 'application/json;version=2'}
        return MDM._get(self, path='/devices/search', header=_header, params=kwargs)

    def searchv3(self, **kwargs):
        """Returns the Device information matching the search parameters with v3 endpoint."""
        _header = {'Accept': 'application/json;version=3'}
        return MDM._get(self, path='/devices/search', header=_header, params=kwargs)

    def search_all(self, **kwargs):
        """Returns the Devices matching the search parameters."""
        response = MDM._get(self, path='/devices/search', params=kwargs)
        return response

    def extensive_search(self, **kwargs):
        """Full device details search with many attributes included.

        PARAMS:
            organizationgroupid (int, optional): OrganizationGroup to be searched,
                user's OG is considered if not sent. Defaults to None.
            platform (str, optional): Device platform. Defaults to None.
            startdatetime (str, optional): Filters devices such that devices with
                last seen after this date will be returned. Defaults to None.
            enddatetime (str, optional): Filters devices such that devices with
                last seen till this date will be returned. Defaults to None.
            deviceid (int, optional): Device Identifier. Defaults to None.
            customattributeslist (str, optional): Custom attribute names.
                Defaults to None.
            enrollmentstatus (str, optional): Filters devices based on EnrollmentStatus
                [Enrolled, EnterpriseWipePending, DeviceWipePending, Unenrolled].
                Defaults to None.
            statuschangestarttime (str, optional): Filters the devices for which
                EnrollmentStatus has changes from enrollmentstatuschangefrom datetime.
                This filter is only for Enrolled and Unenrolled enrollment status.
                Defaults to None.
            statuschangeendtime (str, optional): Filters the devices for which
                EnrollmentStatus has changes till enrollmentstatuschangeto datetime.
                This filter is only for Enrolled and Unenrolled enrollment status.
                Defaults to None.
            page (int, optional): Specific page number to get. 0 based index.
                Defaults to 0.
            pagesize (int, optional): Maximumm records per page. Defaults to 500.
            macaddress (str, optional): MAC address. Defaults to None.

        Returns:
            dict: API paged of devices that meet the search requirements.
        """
        response = MDM._get(
            self, path='/devices/extensivesearch', params=kwargs)
        return response

    def get_details_by_alt_id(self, serialnumber=None, macaddress=None,
                              udid=None, imeinumber=None, easid=None):
        """Returns the Device information matching the search parameters."""
        if serialnumber:
            response = self.search(
                searchby='Serialnumber', id=str(serialnumber))
        elif macaddress:
            response = self.search(searchby='Macaddress', id=str(macaddress))
        elif udid:
            response = self.search(searchby='Udid', id=str(udid))
        elif imeinumber:
            response = self.search(searchby='ImeiNumber', id=str(imeinumber))
        elif easid:
            response = self.search(searchby='EasId', id=str(easid))
        else:
            return None
        return response

    def get_id_by_alt_id(self, serialnumber=None, macaddress=None, udid=None,
                         imeinumber=None, easid=None):
        response = self.get_details_by_alt_id(
            serialnumber, macaddress, udid, imeinumber, easid)
        return response['Id']['Value']

    def clear_device_passcode(self, device_id):
        """
        Clear the passcode on a device
        """
        return MDM._post(self,
                         path='/devices/{}/clearpasscode'.format(device_id))

    def send_commands_for_device_id(self, command, device_id):
        """
        Commands for devices selecting device based on id
        """
        path = '/devices/{}/commands'.format(device_id)
        command = 'command={}'.format(command)
        return MDM._post(self, path=path, params=command)

    def send_commands_by_id(self, command, searchby, id):
        """
        Commands for devices selecting device based on id
        """
        _path = '/devices/commands'
        _query = 'command={}&searchBy={}&id={}'.format(str(command),
                                                       str(searchby),
                                                       str(id))
        return MDM._post(self, path=_path, params=_query)

    def get_details_by_device_id(self, device_id):
        """
        device details by device id
        """
        return MDM._get(self, path='/devices/{}'.format(device_id))

    def get_device_filevault_recovery_key(self, device_uuid):
        """
        Gets a macOS device's FileVault Recovery Key
        """
        _path = '/devices/{}/security/recovery-key'.format(device_uuid)
        return MDM._get(self, path=_path)

    def get_security_info_by_id(self, device_id):
        """
        Processes the device ID to retrieve the security
        information sample related info
        """
        _path = '/devices/{}/security'.format(device_id)
        return MDM._get(self, path=_path)

    def get_security_info_by_alternate_id(self, searchby, id):
        """
        Processes the device ID to retrieve the security
        information sample related info by Alternate ID
        """
        _path = '/devices/security'
        _params = 'searchby={}&id={}'.format(searchby, id)
        return MDM._get(self, path=_path, params=_params)

    def get_bulk_security_info(self, organization_group_id, user_name,
                               params=None):
        """
        Processes the information like organization group ID, user name, model,
        platform, last seen, ownership, compliant status, seen since parameters
        and fetches the security information for the same.
        """
        _path = '/devices/securityinfosearch'
        _query = 'organizationgroupid={}&user={}'.format(organization_group_id,
                                                         user_name)
        return MDM._get(self, path=_path, params=_query)

    def switch_device_from_staging_to_user(self, device_id, user_id):
        """
        API for Single Staging switch to directory or basic user
        """
        _path = "/devices/{}/enrollmentuser/{}".format(device_id, user_id)
        return MDM._patch(self, path=_path)

    def get_managed_admin_account_by_uuid(self, device_id):
        """
        Get information of the administrator account configured on a macOS
        device via device enrollment program (DEP).
        """
        _path = "/devices/{}/security/managed-admin-information".format(
            device_id)
        return MDM._get(self, path=_path)

    def delete_device_by_id(self, device_id):
        """
        Delete a device from management.

        :param device_id: The device ID
        :return: API response
        """
        return MDM._delete(self, path='/devices/{}'.format(device_id))

    def delete_customattribute_by_id(self, device_id, customAttributes):
        """
        Delete a device customattribute.

        :param device_id: The device ID
               customAttributes: The attributes to remove separated by a comma
        :return: API response
        """
        _path = "/devices/{}/customattributes".format(device_id)
        _data = {"CustomAttributes": []}
        for item in customAttributes.split(","):
            _data["CustomAttributes"].append({"Name": item})
        return MDM._delete(self, path=_path, json=_data)

    def delete_customattribute_by_alt_id(self, serialnumber, customAttributes):
        """
        Delete a device customattribute by it's serial number.

        # NOTE: (clayton) This function doesn't seem to work from testing?

        :param device_id: The device ID
               customAttributes: The attributes to remove separated by a comma
        :return: API response
        """
        _path = "/devices/serialnumber/{}/customattributes".format(
            serialnumber)
        _data = {"CustomAttributes": []}
        for item in customAttributes.split(","):
            _data["CustomAttributes"].append({"Name": item})
        return MDM._delete(self, path=_path, json=_data)

    def search_enrollment_token(self, organization_group_uuid, **kwargs):
        """
        Returns a list of enrollment tokes that match the search criteria.

        :param organization_group_uuid: The Uuid of the Organization Group

        :return: API response
        """
        _path = "/groups/{}/enrollment-tokens".format(organization_group_uuid)
        response = MDM._get(self, path=_path, params=kwargs)
        return response

    def create_enrollment_token(self, organization_group_uuid, registration_record):
        """
        Creates a device enrollment token in the given organization unit with a given registration record (data)
        """
        _path = "/groups/{}/enrollment-tokens".format(organization_group_uuid)
        return MDM._post(self, path=_path, json=registration_record)



class MAM(object):
    """
    Base MAM class

    Workspace ONE UEM REST APIs allows you to manage the end-to-end functionalities of Mobile Application Management (MAM) features. 
    Using these APIs, you can upload internal and public applications,
    assign, and manage applications on the devices.
    """

    def __init__(self, client):
        self.client = client

    def _get(self, module='mam', path=None, version=None, params=None,
             header=None):
        """GET requests for the /MAM/blobs module."""
        response = self.client.get(module=module, path=path, version=version,
                                   params=params, header=header)
        return response

    def _post(self, module='mam', path=None, version=None, params=None,
              data=None, json=None, header=None):
        """Post requests for the /MAM/blobs module."""
        response = self.client.post(module=module, path=path, version=version,
                                    params=params, data=data, json=json,
                                    header=header)



import os

from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

# with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
#     README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='pyws1uem',
    version='0.0.7',
    description=('PyWorkspaceOneUEM is a Python API library for VMware Workspace ONE UEM formerly known as AirWatch'),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/marcofuchs89/PyWorkspaceOne',
    author='marcofuchs89',
    author_email='marco@fusche.net',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9',
    install_requires=['requests'],
    keywords='uem airwatch api',
)



from .system import System
import json


class Groups(System):
    """
    A class to manage all core functionalities for AirWatch Organization Groups.
    """
    jheader = {'Content-Type': 'application/json'}

    def __init__(self, client):
        System.__init__(self, client)

    def search(self, **kwargs):
        """Returns the Groups matching the search parameters."""
        response = System._get(self, path='/groups/search', params=kwargs)
        return response

    def get_id_from_groupid(self, groupid):
        """Returns the OG ID for a given Group ID"""
        response = self.search(groupid=str(groupid))
        return response['LocationGroups'][0]['Id']['Value']

    def get_groupid_from_id(self, id):
        """Returns the Group ID for a given ID"""
        response = System._get(self, path='/groups/{}'.format(id))
        return response['GroupId']

    def get_uuid_from_groupid(self, id):
        """Returns the OG UUID for a given Group ID"""
        response = System._get(self, path='/groups/{}'.format(id))
        return response['Uuid']

    def create(self, parent_id, ogdata):
        """Creates a Group and returns the new ID."""
        response = System._post(self, path='/groups/{}'.format(parent_id),
                                data=ogdata, header=self.jheader)
        return response

    def create_customer_og(self, groupid, name=None):
        """
        Creates a Customer type OG, with a given Group ID and Name, and returns
        the new ID
        """
        new_og = {'GroupId': str(groupid),
                  'Name': str(name),
                  'LocationGroupType': 'Customer'}
        if name is None:
            new_og['Name'] = str(groupid)
        response = self.create(parent_id=7, ogdata=json.dumps(new_og))
        return response.get('Value')

    def create_child_og(self, parent_groupid, groupid, og_type=None, name=None):
        """
        Creates a Child OG for a given Parent Group ID, with a given Type,
        Group ID, and Name, and returns the new ID
        """
        pid = self.get_id_from_groupid(parent_groupid)
        new_og = {'GroupId': str(groupid),
                  'Name': str(name),
                  'LocationGroupType': str(og_type)}
        if name is None:
            new_og['Name'] = str(groupid)
        if og_type is None:
            new_og['LocationGroupType'] = 'Container'
        response = self.create(parent_id=pid, ogdata=json.dumps(new_og))
        return response.get('Value')
